//
// IrrKlang Wrapper for Imperative Languages
// Frank Dodd (2006-2009)
//
// This wrapper DLL encapsulates a sub-set of the features of the powerful
// IrrKlang 3D Audio Engine exposing the Object Oriented architecture and
// providing a functional Audio SDK for languages that are not object oriented.
//
// This software was developed using the GCC compiler and Code::Blocks IDE
//

/* ////////////////////////////////////////////////////////////////////////////
includes
*/

#include <stdio.h>
#include "irrklangwrapper.h"


/* ////////////////////////////////////////////////////////////////////////////
global variables
*/

/* ////////////////////////////////////////////////////////////////////////////
external variables
*/
extern ISoundEngine* engine;


/* ////////////////////////////////////////////////////////////////////////////
Global Function Declarations

all of the below functions are declared as C functions and are exposed without
any mangled names so that they can be easily imported into imperative
languages like FreeBasic
*/
extern "C"
{

/* ////////////////////////////////////////////////////////////////////////////
SYSTEM FUNCTIONS
*/
/* ----------------------------------------------------------------------------
only one irrklang device object is supported by this wrapper. this call creates
the device object that is exposed to FreeBasic
*/
bool DLL_EXPORT IrrKlangStart( void )
{
	bool boInitStatus = true;
	// start the sound engine with default parameters
	engine = createIrrKlangDevice();

	if (!engine)
	{
		boInitStatus = false; // error starting up the engine
	}
    return boInitStatus;
}


/* ----------------------------------------------------------------------------
Update the IrrKlang system when running in single threaded mode
*/
void DLL_EXPORT IrrKlangUpdate ( )
{
    // updated the irrklang system when running in single threaded mode
    engine->update();
}


/* ----------------------------------------------------------------------------
Set the global sound volume
*/
void DLL_EXPORT IrrKlangSetSoundVolume ( float volume )
{
    // end the scene and display it
    engine->setSoundVolume( volume );
}


/* ----------------------------------------------------------------------------
Play a 2D sound
*/
ISound * DLL_EXPORT IrrKlangPlay2D (
								const char * file,
								unsigned int playLooped,
								unsigned int startPaused,
								unsigned int track,
								E_STREAM_MODE streamMode,
								unsigned int enableSoundEffects )
{
	return engine->play2D( file, playLooped, startPaused, track, streamMode,enableSoundEffects );
}


/* ----------------------------------------------------------------------------
Play a 3D sound
*/
ISound * DLL_EXPORT IrrKlangPlay3D (
								const char * file,
								float posX,
								float posY,
								float posZ,
								unsigned int playLooped,
								unsigned int startPaused,
								unsigned int track,
								E_STREAM_MODE streamMode,
								unsigned int enableSoundEffects )
{
	return engine->play3D( file, vec3df(posX, posY, posZ), playLooped, startPaused, track, streamMode,enableSoundEffects );
}


/* ----------------------------------------------------------------------------
Pause all sounds
*/
void DLL_EXPORT IrrKlangSetAllSoundsPaused ( bool pause )
{
	// pause all sounds
	engine->setAllSoundsPaused( pause );
}


/* ----------------------------------------------------------------------------
Stop all sounds
*/
void DLL_EXPORT IrrKlangStopAllSounds ()
{
	// pause all sounds
	engine->stopAllSounds();
}


/* ----------------------------------------------------------------------------
Set the position of the listener for 3D sounds
*/
void DLL_EXPORT IrrKlangSetListenerPosition (
												float posX,
												float posY,
												float posZ,
												float lookX,
												float lookY,
												float lookZ,
												float velocityX,
												float velocityY,
												float velocityZ,
												float upX,
												float upY,
												float upZ )
{
    // draw the gui interface
    engine->setListenerPosition  (
			vec3df( posX, posY, posZ ),
			vec3df( lookX, lookY, lookZ ),
			vec3df( velocityX, velocityY, velocityZ ),
			vec3df( upX, upY, upZ));
}


/* ----------------------------------------------------------------------------
Release a sound
*/
void DLL_EXPORT IrrReleaseSound ( ISound * sound )
{
    // release the sound resourse
    sound->drop();
}


/* ----------------------------------------------------------------------------
Remove a specific sound file from memory
*/
void DLL_EXPORT IrrRemoveSoundSource ( const char * file )
{
    // remove a specific sound file from memory
    engine->removeSoundSource( file );
}


/* ----------------------------------------------------------------------------
Stop all sounds and remove all sound files from memory
*/
void DLL_EXPORT IrrKlangRemoveAllSoundSources ( void )
{
    // begin the scene
	engine->removeAllSoundSources();
}


/* ----------------------------------------------------------------------------
Stop the sound engine
*/
void DLL_EXPORT IrrKlangStop( void )
{
	engine->drop();
}

//Returns amount of loaded sound source
unsigned int DLL_EXPORT IrrKlangGetSoundSourceCount( void )
{
  return engine->getSoundSourceCount();
}

//Returns master sound volume
float DLL_EXPORT IrrKlangGetSoundVolume (void)
{
   return engine->getSoundVolume();
}

//Returns if a sound with the specified name is currently playing
void DLL_EXPORT IrrIsCurrentlyPlaying ( const char * file )
{
   engine->isCurrentlyPlaying( file );
}

//Returns if irrKlang is running in the same thread as the application or is using multithreading
void DLL_EXPORT IrrKlangIsMultiThreaded()
{
  engine->isMultiThreaded();
}

//Sets the default minimal distance for 3D sounds
void DLL_EXPORT IrrKlangSetDefault3DSoundMinDistance(float distance )
{
    engine->setDefault3DSoundMinDistance( distance );
}

//Returns the default minimal distance for 3D sounds
float DLL_EXPORT IrrKlangGetDefault3DSoundMinDistance(void)
{
    return engine->getDefault3DSoundMinDistance();
}

//Sets the default maximal distance for 3D sounds
void DLL_EXPORT IrrKlangSetDefault3DSoundMaxDistance(float distance )
{
    engine->setDefault3DSoundMaxDistance( distance );
}

//Returns the default maximal distance for 3D sounds
float DLL_EXPORT IrrKlangGetDefault3DSoundMaxDistance(void)
{
    return engine->getDefault3DSoundMaxDistance();
}

//Sets a rolloff factor which influences the amount of attenuation that is applied to 3D sounds
void DLL_EXPORT IrrKlangSetRolloffFactor ( float rolloff )
{
    engine->setRolloffFactor( rolloff );
}

//Sets parameters affecting the doppler effect
void DLL_EXPORT IrrKlangSetDopplerEffectParameters ( float dopplerFactor,float distanceFactor )
{
    engine->setDopplerEffectParameters( dopplerFactor,distanceFactor );
}

/* ////////////////////////////////////////////////////////////////////////////
all of the above functions are declared as C functions and are exposed without
any mangled names
*/
}
